;
; TryCatch Library for Purebasic
;-----------------------------------------------------------------------------
;   (c) 2002/2005 Sebastian Lackner 
;
;     This library is free software; you can redistribute it and/or
;     modify it under the terms of the GNU Lesser General Public
;     License as published by the Free Software Foundation; either
;     version 2.1 of the License, Or (at your option) any later version.
; 
;     This library is distributed in the hope that it will be useful,
;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;     MERCHANTABILITY Or FITNESS For A PARTICULAR PURPOSE.  See the GNU
;     Lesser General Public License For more details. 
;----------------------------------------------------------------------------- 


;Imports from Kernel32
EXTERN _SetUnhandledExceptionFilter@4
EXTERN _GlobalAlloc@8
EXTERN _GlobalFree@4
EXTERN _RaiseException@16
EXTERN _PB_DEBUGGER_LineNumber
EXTERN _PB_DEBUGGER_IncludedFiles
;EXTERN _SYS_GetOutputBufferVirtual@4
;EXTERN _PB_StringBasePosition
;EXTERN _SYS_CopyString8@0
;EXTERN PB_StringBase
EXTERN _SYS_GetOutputBuffer@8 ;(int Length, int PreviousPosition);
EXTERN _strcpy@8
EXTERN _strlen@4


%macro PBFunction 1
  Global PB_%1
PB_%1:
%endmacro

%macro Beep 2
  PUSH dword %2
  PUSH dword %1
  EXTERN _Beep@8
  call _Beep@8
%endmacro

SEGMENT .text USE32 CLASS=CODE

TopLevelErrorHandler:
  MOV eax,dword[esp+4]
  MOV ecx,dword[eax] ; pException Record
  MOV ecx,dword[ecx] ; Exception Code
  MOV dword[ExceptionCode],ecx
  MOV eax,dword[eax+4] ; pContext Record
  MOV dword[eax+184], ErrorHandler
  MOV eax,-1
  RET 4


ErrorHandler: ; keine richtige funktion mit Ret-Adresse
  MOV ecx, dword[TryStartMemory]
  MOV esp, [ecx+8] ; ESP wiederherstellen
  
  LEA eax, [TryStartMemory]
  CALL GetElemValue
  MOV ecx, eax
  
  MOV eax, dword[_PB_DEBUGGER_LineNumber] 
  CMP eax, -1
  JE .nolinenb
    AND eax, 0xFFFFFF ; Zeilennummer
    MOV dword[ExceptionLine], eax
    
    MOV eax, dword[_PB_DEBUGGER_LineNumber] 
    AND eax, 0xFF000000 ; IncludeFile
    SHR eax, 24
    MOV dword[ExceptionIncludeFile], eax
    JMP .ok
  .nolinenb:
    MOV dword[ExceptionLine], -1
    MOV dword[ExceptionIncludeFile], 0
  .ok:
  
  MOV eax,0 ; um in den Else-Zweig zu kommen
  JMP dword ecx
 



;########################### PB Functions

PBFunction GetTryErrorCode
GetTryCode:
  MOV eax, dword[ExceptionCode]
  RET

PBFunction GetTryErrorLine
GetTryErrorLine:
  MOV eax, dword[ExceptionLine]
  RET

PBFunction GetTryErrorIncludeFile
GetTryErrorIncludeFile:
  MOV eax, dword[ExceptionIncludeFile]
  CMP eax, 0
  JNE .normal
    LEA eax, [s_MainFile]
    JMP .end
  .normal:
    DEC eax
    LEA eax, [_PB_DEBUGGER_IncludedFiles + eax * 4]
    MOV eax, dword[eax]
  .end:
  
  PUSH eax ; Save
  PUSH eax ; argument String
  CALL _strlen@4
  ADD esp, 4
  ; [esp] = String, eax = len
  
  PUSH dword[esp+8] ; PreviousPosition  (offset: String + Ret)
  PUSH eax ; NeededLen
  CALL _SYS_GetOutputBuffer@8
  ; SourceString in [Esp]
  PUSH eax ; Destination
  CALL _strcpy@8
  ADD esp, 8
  
  RET ;4



PBFunction SetTryErrorCode
SetTryCode:
  PUSH dword 0
  PUSH dword 0
  PUSH dword 0
  PUSH dword eax ; Param: Code
  CALL _RaiseException@16
  RET 4


PBFunction EndTryError
EndTry:
  LEA eax, [TryLastStartMemory]
  CALL GetElemValue
  PUSH dword eax
  CALL   _SetUnhandledExceptionFilter@4 ; zurcksetzen
  
  LEA edx, [TryStartMemory]
  CALL DelElem
  LEA edx, [TryLastStartMemory]
  CALL DelElem
  XOR eax, eax
  RET



PBFunction TryError
Try:
  LEA ecx, [TryStartMemory]
  POP edx ; speichern der Try-Startadresse
  MOV eax, esp ; ESP vor Funktionsaufruf!
  PUSH edx
  PUSH edi
  MOV edi, eax ; ESP
  CALL AddElem
  POP edi
  
  PUSH dword TopLevelErrorHandler
  CALL _SetUnhandledExceptionFilter@4

  LEA ecx, [TryLastStartMemory]
  MOV edx, eax ; speichern der Try-LastStartadresse
  CALL AddElem

  MOV eax,1
  RET


;#################### String ####################
; eax, ecx, edx
CopyString: ; esp=ret, esp+4=toString, esp+8=FromString
  MOV ecx, dword[esp+4] ; To
  MOV eax, dword[esp+8] ; From
  XOR edx, edx

  .loop:
  MOV dl, byte[eax]
  CMP edx, 0
  JE .end
    MOV byte[ecx], dl
    INC ecx
    INC eax
    JMP .loop
  .end:
  MOV byte[ecx], 0
  MOV eax, dword[esp+4]
  RET 8



;StrLen: ;esp=ret
;  MOV ecx, eax ; String
;  XOR eax, eax ; len
;
;  .loop:
;  CMP byte[ecx], 0
;  JE .end
;    INC ecx
;    INC eax
;    JMP .loop
;  .end:
;  RET




;###################### LinkedList / STACK
AddElem: ; Ecx: Addr. to CurrElemVar,    Edx: Value     Edi:ESP to store
  PUSH ebx
  PUSH ecx
  PUSH edx
  PUSH dword 12
  PUSH dword 0
  CALL _GlobalAlloc@8
  POP edx
  POP ecx
  MOV ebx, dword[ecx] ; Addr. to CurrElem
  MOV dword[eax], ebx ; last elem = CurrElem
  MOV dword[eax+4], edx ; value
  MOV dword[eax+8], edi ; esp
  MOV dword[ecx], eax ; CurrElem = Allocated Elem
  POP ebx
  RET



DelElem: ; Edx: Addr. to CurrElemVar
  MOV eax, dword[edx] ; Addr. to CurrElem
  MOV ecx, dword[eax] ; last elem
  MOV dword[edx], ecx ; CurrElem = last elem
  PUSH dword eax
  CALL _GlobalFree@4
  RET



GetElemValue: ; Eax: Addr. to CurrElemVar
  POP ecx ; RET
  MOV eax, dword[eax]
  CMP eax, 0
  JE .null
    MOV eax, dword[eax+4] ; Value
    JMP .got
  .null
    MOV eax, 0
  .got
  JMP ecx







SEGMENT .data CLASS=DATA
s_MainFile: db 'MainFile', 0

SEGMENT .bss CLASS=DATA
ALIGNB 4
TryStartMemory: RESD 1
TryLastStartMemory: RESD 1
ExceptionCode: RESD 1
ExceptionLine: RESD 1
ExceptionIncludeFile: RESD 1

I_BSSEnd
