/*
 *
 *  Copyright (c) 2002-2005 by Danilo Krahn
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, Or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY Or FITNESS For A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License For more details. 
 *
 */
#include "SkinWinFast.h"



PBFUNCTION(RGNDATA*) PB_CatchFastSkin(_sw_DKPS *skin) {
   if(skin) {
      void *FinalMemory;

      // check signature "DKPS"
      if(skin->signature != 0x53504B44) return 0;
      // check version
      if(skin->version != 1) return 0;
      // check header size
      if(skin->size_of_header != sizeof(_sw_DKPS)) return 0;
      

      //check for compression flag
      if(skin->compressed) {
         // compressed skin -> uncompress now
         // allocate memory
         FinalMemory = PB_AllocateMemory(skin->size_of_uncompressed_data);
         // alloc failed, return
         if(!FinalMemory) return 0;
         // unpack
         if(!PB_UnpackMemory((void*)((unsigned long)skin + skin->size_of_header),FinalMemory)) {
            // uncompressing failed
            PB_FreeMemory(FinalMemory);
            return 0;
         }
      }
      else {
         // uncompressed skin
         FinalMemory = (void*)((unsigned long)skin + skin->size_of_header);
      }

      // memory ready - create FastSkin
      long index;
      if((index = sub_swGetFastSkinArea()) == -1) {
         // failed
         if(skin->compressed) PB_FreeMemory(FinalMemory);
         return 0;
      }
    
      sw_FastSkinArea[index].region_data      = PB_AllocateMemory(skin->size_of_uncompressed_region_data);
      if(!sw_FastSkinArea[index].region_data) {
         // failed
         if(skin->compressed) PB_FreeMemory(FinalMemory);
         return 0;
      }


      sw_FastSkinArea[index].region_data_size = skin->size_of_uncompressed_region_data;
      PB_CopyMemory((void*)((unsigned long)FinalMemory + skin->region_offset),
                    sw_FastSkinArea[index].region_data,
                    sw_FastSkinArea[index].region_data_size);

      if(skin->size_of_uncompressed_bitmap) {
  
         // get desktop DC
         HDC DesktopDC  = GetDC(GetDesktopWindow());
         if(!DesktopDC) {
            // failed
            if(skin->compressed) PB_FreeMemory(FinalMemory);
            PB_FreeMemory(sw_FastSkinArea[index].region_data);
            sw_FastSkinArea[index].region_data      = 0;
            sw_FastSkinArea[index].region_data_size = 0;
            return 0;
         }

         // create bitmap
         BITMAPFILEHEADER *bfh = (BITMAPFILEHEADER*)((unsigned long)FinalMemory + skin->bitmap_offset);
         BITMAPINFO       *bi  = (BITMAPINFO*)      ((unsigned long)bfh         + sizeof(BITMAPFILEHEADER));

         if(!(sw_FastSkinArea[index].hBmp = CreateCompatibleBitmap(DesktopDC,bi->bmiHeader.biWidth,bi->bmiHeader.biHeight))) {
            // failed
            if(skin->compressed) PB_FreeMemory(FinalMemory);
            PB_FreeMemory(sw_FastSkinArea[index].region_data);
            sw_FastSkinArea[index].region_data      = 0;
            sw_FastSkinArea[index].region_data_size = 0;
            ReleaseDC(GetDesktopWindow(),DesktopDC);
            return 0;
         }
        
         //release desktop DC
         ReleaseDC(GetDesktopWindow(),DesktopDC);
  
         // copy bitmap data
         HDC hDC = CreateCompatibleDC(0);
           //HBITMAP oldBmp = (HBITMAP)SelectObject(dc,sw_FastSkinArea[index].hBmp);
           SetDIBits(hDC,sw_FastSkinArea[index].hBmp,0,bi->bmiHeader.biHeight,(void*)((unsigned long)bfh + bfh->bfOffBits),bi,DIB_RGB_COLORS);
           //SelectObject(dc,oldBmp);
         DeleteDC(hDC);
      }
      else sw_FastSkinArea[index].hBmp = 0; // no bitmap included

      // set skin width & height
      sw_FastSkinArea[index].skin_width  = skin->skin_width;
      sw_FastSkinArea[index].skin_height = skin->skin_height;

      // return success
      return sw_FastSkinArea[index].region_data;
   }
   return 0;
}

