/*
 *
 *  Copyright (c) 2002-2005 by Danilo Krahn
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, Or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY Or FITNESS For A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License For more details. 
 *
 */
#include <PBOSL.h>
#include <windows.h>
#include <commctrl.h>



PBFUNCTION(unsigned long) PB_SetTBimage(HBITMAP hImage,
                                        HBITMAP hMask,
                                        unsigned long type) {
   if(PBOSL_TBpro_Current.TB) {
      HIMAGELIST list;
      // create the image list
      if(list = ImageList_Create(PBOSL_TBpro_Current.ButtonSizeX,
                                 PBOSL_TBpro_Current.ButtonSizeY,
                                 ILC_COLORDDB|ILC_MASK,
                                 0,
                                 1000)) {


         // if hMask is 0, create a mask
         int mask_created = 0;
         if(!hMask && hImage) {
            // get size of hImage
            BITMAP bmp;
            GetObject(hImage,sizeof(BITMAP),&bmp);
            // create DC and BMP, both compatible to screen
            HDC DDC = CreateDC("DISPLAY",0,0,0);
            HDC hDC = CreateCompatibleDC(DDC);
            hMask   = CreateCompatibleBitmap(DDC,bmp.bmWidth,bmp.bmHeight);
            DeleteDC(DDC);
            
            // select new image into DC and fill it with black for the mask
            HBITMAP oldBmp = SelectObject(hDC,hMask);
              RECT rect;
              rect.left   = 0;
              rect.top    = 0;
              rect.right  = bmp.bmWidth+1;
              rect.bottom = bmp.bmHeight+1;
              FillRect(hDC,&rect,GetStockObject(BLACK_BRUSH));
            SelectObject(hDC,oldBmp);
            DeleteDC(hDC);
            mask_created = 1;
         }
         
         // add image and mask to imagelist
         if(ImageList_Add(list,hImage,hMask) == -1) {
            ImageList_Destroy(list);
            return 0;
         }
         
         // delete mask image, if created automatically
         if(mask_created) DeleteObject(hMask);
         
         // main image
         if(type == TB_SETIMAGELIST) {
            PBOSL_TBpro_Current.ImageCount = ImageList_GetImageCount(list);
            PBOSL_TBpro_Current.hList      = list;
         }
         // hot item and disabled item
         else {
            if(PBOSL_TBpro_Current.ImageCount != ImageList_GetImageCount(list)) {
               ImageList_Destroy(list);
               return 0;
            }
            if(type == TB_SETHOTIMAGELIST)
               PBOSL_TBpro_Current.hList_hot      = list;
            if(type == TB_SETDISABLEDIMAGELIST)
               PBOSL_TBpro_Current.hList_disabled = list;
         }

         // sync handle-area with current TB
         // (updates ImageCount and imagelist handles)
         for(long i=0; i<PBOSL_TBpro_Handles_Size; i++) {
            if(PBOSL_TBpro_Handles[i].Number == PBOSL_TBpro_Current.Number) {
               PBOSL_TBpro_Handles[i] = PBOSL_TBpro_Current;
            }
         }

         // set image for toolbar
         ImageList_Destroy((HIMAGELIST)SendMessage(PBOSL_TBpro_Current.TB,
                                                   type,0,(LPARAM)list));
         
         //
         // set button size again and update TB -> required for Win9x!!!!
         //
         // error description: if this workaround is not applied,
         //                    the button height is not always
         //                    returned correctly on Win9x systems!
         //
         SendMessage(PBOSL_TBpro_Current.TB,TB_SETBITMAPSIZE ,0,(LPARAM)(DWORD)MAKELONG(PBOSL_TBpro_Current.ButtonSizeX,PBOSL_TBpro_Current.ButtonSizeY));
         SendMessage(PBOSL_TBpro_Current.TB,TB_AUTOSIZE,0,0);

         // return success
         return 1;
      }
   }
   return 0; // error
}
