; ----------------------
; SQLite3
; by Peter Tuebben (aka Kiffi)
; ----------------------
CompilerIf Defined(SQLITE3_Defined, #PB_Constant) = #False

  #SQLITE3_Defined = #True

  Structure SQLITE3_RecordSet
    BOF.l
    EOF.l
    Handle.l
    Rows.l
    Cols.l
    CurrentPos.l
    sValue.s
  EndStructure

  ; Constants

  #SQLITE3_OK          =   0   ; Successful Result
  #SQLITE3_ERROR       =   1   ; SQL error Or missing database
  #SQLITE3_INTERNAL    =   2   ; An internal logic error in SQLite
  #SQLITE3_PERM        =   3   ; Access permission denied
  #SQLITE3_ABORT       =   4   ; Callback routine requested An abort
  #SQLITE3_BUSY        =   5   ; The database file is locked
  #SQLITE3_LOCKED      =   6   ; A table in The database is locked
  #SQLITE3_NOMEM       =   7   ; A malloc() failed
  #SQLITE3_READONLY    =   8   ; Attempt To write A readonly database
  #SQLITE3_INTERRUPT   =   9   ; Operation terminated by SQLite_Interrupt()
  #SQLITE3_IOERR       =  10   ; Some kind of disk I/O error occurred
  #SQLITE3_CORRUPT     =  11   ; The database disk image is malformed
  #SQLITE3_NOTFOUND    =  12   ; (internal Only) table Or record not found
  #SQLITE3_FULL        =  13   ; Insertion failed because database is full
  #SQLITE3_CANTOPEN    =  14   ; Unable To open The database file
  #SQLITE3_PROTOCOL    =  15   ; database lock protocol error
  #SQLITE3_EMPTY       =  16   ; (internal Only) database table is empty
  #SQLITE3_SCHEMA      =  17   ; The database schema changed
  #SQLITE3_TOOBIG      =  18   ; Too much Data For one Row of A table
  #SQLITE3_CONStraint  =  19   ; abort due To contraint violation
  #SQLITE3_MISMATCH    =  20   ; Data type mismatch
  #SQLITE3_MISUSE      =  21   ; Library used incorrectly
  #SQLITE3_NOLFS       =  22   ; Uses OS features not supported on host
  #SQLITE3_AUTH        =  23   ; Authorization denied
  #SQLITE3_ROW         = 100   ; sqlite_step() has another Row ready
  #SQLITE3_DONE        = 101   ; sqlite_step() has finished executing

  #SQLITE3_STATIC = 0

  ; Column-Datatypes:

  #SQLITE_INTEGER = 1
  #SQLITE_FLOAT   = 2
  #SQLITE_TEXT    = 3
  #SQLITE_BLOB    = 4
  #SQLITE_NULL    = 5

CompilerEndIf

#ERROR_STATEMENT_IS_EMPTY$ = "Statement is empty"

;- Declarations

Declare   SetLastMessage(hResult)

;{ Globals

Global LastMessage.s
Global LibIsInitializedWithIncludedDLL.l

Global sqlite3_aggregate_context.l
Global sqlite3_aggregate_count.l
Global sqlite3_bind_blob.l
Global sqlite3_bind_double.l
Global sqlite3_bind_int.l
Global sqlite3_bind_int64.l
Global sqlite3_bind_null.l
Global sqlite3_bind_parameter_count.l
Global sqlite3_bind_parameter_index.l
Global sqlite3_bind_parameter_name.l
Global sqlite3_bind_text.l
Global sqlite3_bind_text16.l
Global sqlite3_busy_handler.l
Global sqlite3_busy_timeout.l
Global sqlite3_changes.l
Global sqlite3_close.l
Global sqlite3_collation_needed.l
Global sqlite3_collation_needed16.l
Global sqlite3_column_blob.l
Global sqlite3_column_bytes.l
Global sqlite3_column_bytes16.l
Global sqlite3_column_count.l
Global sqlite3_column_decltype.l
Global sqlite3_column_decltype16.l
Global sqlite3_column_double.l
Global sqlite3_column_int.l
Global sqlite3_column_int64.l
Global sqlite3_column_name.l
Global sqlite3_column_name16.l
Global sqlite3_column_text.l
Global sqlite3_column_text16.l
Global sqlite3_column_type.l
Global sqlite3_commit_hook.l
Global sqlite3_complete.l
Global sqlite3_complete16.l
Global sqlite3_create_collation.l
Global sqlite3_create_collation16.l
Global sqlite3_create_function.l
Global sqlite3_create_function16.l
Global sqlite3_data_count.l
Global sqlite3_errcode.l
Global sqlite3_errmsg.l
Global sqlite3_errmsg16.l
Global sqlite3_exec.l
Global sqlite3_finalize.l
Global sqlite3_free.l
Global sqlite3_free_table.l
Global sqlite3_get_auxdata.l
Global sqlite3_get_table.l
Global sqlite3_interrupt.l
Global sqlite3_last_insert_rowid.l
Global sqlite3_libversion.l
Global sqlite3_mprintf.l
Global sqlite3_open.l
Global sqlite3_open16.l
Global sqlite3_prepare.l
Global sqlite3_prepare16.l
Global sqlite3_progress_handler.l
Global sqlite3_reset.l
Global sqlite3_result_blob.l
Global sqlite3_result_double.l
Global sqlite3_result_error.l
Global sqlite3_result_error16.l
Global sqlite3_result_int.l
Global sqlite3_result_int64.l
Global sqlite3_result_null.l
Global sqlite3_result_text.l
Global sqlite3_result_text16.l
Global sqlite3_result_text16be.l
Global sqlite3_result_text16le.l
Global sqlite3_result_value.l
Global sqlite3_set_authorizer.l
Global sqlite3_set_auxdata.l
Global sqlite3_snprintf.l
Global sqlite3_step.l
Global sqlite3_total_changes.l
Global sqlite3_trace.l
Global sqlite3_user_data.l
Global sqlite3_value_blob.l
Global sqlite3_value_bytes.l
Global sqlite3_value_bytes16.l
Global sqlite3_value_double.l
Global sqlite3_value_int.l
Global sqlite3_value_int64.l
Global sqlite3_value_text.l
Global sqlite3_value_text16.l
Global sqlite3_value_text16be.l
Global sqlite3_value_text16le.l
Global sqlite3_value_type.l
Global sqlite3_vmprintf.l

;// Globale Variable der DLL
Global sqlite3DLL

;}

ProcedureDLL.l SQLite3_End()
  ;// Release DLL
  If sqlite3DLL
    If LibIsInitializedWithIncludedDLL
      FreeLibraryM(sqlite3DLL)
    Else
      FreeLibrary_(sqlite3DLL)
    EndIf
  EndIf
  sqlite3DLL = 0
EndProcedure

ProcedureDLL.l SQLite3_InitLib(sSQLiteDLLPath.s)

  If sqlite3DLL
    SQLite3_End()
  EndIf

  sqlite3DLL = LoadLibrary_(@sSQLiteDLLPath)

  If sqlite3DLL

    sqlite3_aggregate_context     = GetProcAddress_(sqlite3DLL, "sqlite3_aggregate_context")
    sqlite3_aggregate_count       = GetProcAddress_(sqlite3DLL, "sqlite3_aggregate_count")
    sqlite3_bind_blob             = GetProcAddress_(sqlite3DLL, "sqlite3_bind_blob")
    sqlite3_bind_double           = GetProcAddress_(sqlite3DLL, "sqlite3_bind_double")
    sqlite3_bind_int              = GetProcAddress_(sqlite3DLL, "sqlite3_bind_int")
    sqlite3_bind_int64            = GetProcAddress_(sqlite3DLL, "sqlite3_bind_int64")
    sqlite3_bind_null             = GetProcAddress_(sqlite3DLL, "sqlite3_bind_null")
    sqlite3_bind_parameter_count  = GetProcAddress_(sqlite3DLL, "sqlite3_bind_parameter_count")
    sqlite3_bind_parameter_index  = GetProcAddress_(sqlite3DLL, "sqlite3_bind_parameter_index")
    sqlite3_bind_parameter_name   = GetProcAddress_(sqlite3DLL, "sqlite3_bind_parameter_name")
    sqlite3_bind_text             = GetProcAddress_(sqlite3DLL, "sqlite3_bind_text")
    sqlite3_bind_text16           = GetProcAddress_(sqlite3DLL, "sqlite3_bind_text16")
    sqlite3_busy_handler          = GetProcAddress_(sqlite3DLL, "sqlite3_busy_handler")
    sqlite3_busy_timeout          = GetProcAddress_(sqlite3DLL, "sqlite3_busy_timeout")
    sqlite3_changes               = GetProcAddress_(sqlite3DLL, "sqlite3_changes")
    sqlite3_close                 = GetProcAddress_(sqlite3DLL, "sqlite3_close")
    sqlite3_collation_needed      = GetProcAddress_(sqlite3DLL, "sqlite3_collation_needed")
    sqlite3_collation_needed16    = GetProcAddress_(sqlite3DLL, "sqlite3_collation_needed16")
    sqlite3_column_blob           = GetProcAddress_(sqlite3DLL, "sqlite3_column_blob")
    sqlite3_column_bytes          = GetProcAddress_(sqlite3DLL, "sqlite3_column_bytes")
    sqlite3_column_bytes16        = GetProcAddress_(sqlite3DLL, "sqlite3_column_bytes16")
    sqlite3_column_count          = GetProcAddress_(sqlite3DLL, "sqlite3_column_count")
    sqlite3_column_decltype       = GetProcAddress_(sqlite3DLL, "sqlite3_column_decltype")
    sqlite3_column_decltype16     = GetProcAddress_(sqlite3DLL, "sqlite3_column_decltype16")
    sqlite3_column_double         = GetProcAddress_(sqlite3DLL, "sqlite3_column_double")
    sqlite3_column_int            = GetProcAddress_(sqlite3DLL, "sqlite3_column_int")
    sqlite3_column_int64          = GetProcAddress_(sqlite3DLL, "sqlite3_column_int64")
    sqlite3_column_name           = GetProcAddress_(sqlite3DLL, "sqlite3_column_name")
    sqlite3_column_name16         = GetProcAddress_(sqlite3DLL, "sqlite3_column_name16")
    sqlite3_column_text           = GetProcAddress_(sqlite3DLL, "sqlite3_column_text")
    sqlite3_column_text16         = GetProcAddress_(sqlite3DLL, "sqlite3_column_text16")
    sqlite3_column_type           = GetProcAddress_(sqlite3DLL, "sqlite3_column_type")
    sqlite3_commit_hook           = GetProcAddress_(sqlite3DLL, "sqlite3_commit_hook")
    sqlite3_complete              = GetProcAddress_(sqlite3DLL, "sqlite3_complete")
    sqlite3_complete16            = GetProcAddress_(sqlite3DLL, "sqlite3_complete16")
    sqlite3_create_collation      = GetProcAddress_(sqlite3DLL, "sqlite3_create_collation")
    sqlite3_create_collation16    = GetProcAddress_(sqlite3DLL, "sqlite3_create_collation16")
    sqlite3_create_function       = GetProcAddress_(sqlite3DLL, "sqlite3_create_function")
    sqlite3_create_function16     = GetProcAddress_(sqlite3DLL, "sqlite3_create_function16")
    sqlite3_data_count            = GetProcAddress_(sqlite3DLL, "sqlite3_data_count")
    sqlite3_errcode               = GetProcAddress_(sqlite3DLL, "sqlite3_errcode")
    sqlite3_errmsg                = GetProcAddress_(sqlite3DLL, "sqlite3_errmsg")
    sqlite3_errmsg16              = GetProcAddress_(sqlite3DLL, "sqlite3_errmsg16")
    sqlite3_exec                  = GetProcAddress_(sqlite3DLL, "sqlite3_exec")
    sqlite3_finalize              = GetProcAddress_(sqlite3DLL, "sqlite3_finalize")
    sqlite3_free                  = GetProcAddress_(sqlite3DLL, "sqlite3_free")
    sqlite3_free_table            = GetProcAddress_(sqlite3DLL, "sqlite3_free_table")
    sqlite3_get_auxdata           = GetProcAddress_(sqlite3DLL, "sqlite3_get_auxdata")
    sqlite3_get_table             = GetProcAddress_(sqlite3DLL, "sqlite3_get_table")
    sqlite3_interrupt             = GetProcAddress_(sqlite3DLL, "sqlite3_interrupt")
    sqlite3_last_insert_rowid     = GetProcAddress_(sqlite3DLL, "sqlite3_last_insert_rowid")
    sqlite3_libversion            = GetProcAddress_(sqlite3DLL, "sqlite3_libversion")
    sqlite3_mprintf               = GetProcAddress_(sqlite3DLL, "sqlite3_mprintf")
    sqlite3_open                  = GetProcAddress_(sqlite3DLL, "sqlite3_open")
    sqlite3_open16                = GetProcAddress_(sqlite3DLL, "sqlite3_open16")
    sqlite3_prepare               = GetProcAddress_(sqlite3DLL, "sqlite3_prepare")
    sqlite3_prepare16             = GetProcAddress_(sqlite3DLL, "sqlite3_prepare16")
    sqlite3_progress_handler      = GetProcAddress_(sqlite3DLL, "sqlite3_progress_handler")
    sqlite3_reset                 = GetProcAddress_(sqlite3DLL, "sqlite3_reset")
    sqlite3_result_blob           = GetProcAddress_(sqlite3DLL, "sqlite3_result_blob")
    sqlite3_result_double         = GetProcAddress_(sqlite3DLL, "sqlite3_result_double")
    sqlite3_result_error          = GetProcAddress_(sqlite3DLL, "sqlite3_result_error")
    sqlite3_result_error16        = GetProcAddress_(sqlite3DLL, "sqlite3_result_error16")
    sqlite3_result_int            = GetProcAddress_(sqlite3DLL, "sqlite3_result_int")
    sqlite3_result_int64          = GetProcAddress_(sqlite3DLL, "sqlite3_result_int64")
    sqlite3_result_null           = GetProcAddress_(sqlite3DLL, "sqlite3_result_null")
    sqlite3_result_text           = GetProcAddress_(sqlite3DLL, "sqlite3_result_text")
    sqlite3_result_text16         = GetProcAddress_(sqlite3DLL, "sqlite3_result_text16")
    sqlite3_result_text16be       = GetProcAddress_(sqlite3DLL, "sqlite3_result_text16be")
    sqlite3_result_text16le       = GetProcAddress_(sqlite3DLL, "sqlite3_result_text16le")
    sqlite3_result_value          = GetProcAddress_(sqlite3DLL, "sqlite3_result_value")
    sqlite3_set_authorizer        = GetProcAddress_(sqlite3DLL, "sqlite3_set_authorizer")
    sqlite3_set_auxdata           = GetProcAddress_(sqlite3DLL, "sqlite3_set_auxdata")
    sqlite3_snprintf              = GetProcAddress_(sqlite3DLL, "sqlite3_snprintf")
    sqlite3_step                  = GetProcAddress_(sqlite3DLL, "sqlite3_step")
    sqlite3_total_changes         = GetProcAddress_(sqlite3DLL, "sqlite3_total_changes")
    sqlite3_trace                 = GetProcAddress_(sqlite3DLL, "sqlite3_trace")
    sqlite3_user_data             = GetProcAddress_(sqlite3DLL, "sqlite3_user_data")
    sqlite3_value_blob            = GetProcAddress_(sqlite3DLL, "sqlite3_value_blob")
    sqlite3_value_bytes           = GetProcAddress_(sqlite3DLL, "sqlite3_value_bytes")
    sqlite3_value_bytes16         = GetProcAddress_(sqlite3DLL, "sqlite3_value_bytes16")
    sqlite3_value_double          = GetProcAddress_(sqlite3DLL, "sqlite3_value_double")
    sqlite3_value_int             = GetProcAddress_(sqlite3DLL, "sqlite3_value_int")
    sqlite3_value_int64           = GetProcAddress_(sqlite3DLL, "sqlite3_value_int64")
    sqlite3_value_text            = GetProcAddress_(sqlite3DLL, "sqlite3_value_text")
    sqlite3_value_text16          = GetProcAddress_(sqlite3DLL, "sqlite3_value_text16")
    sqlite3_value_text16be        = GetProcAddress_(sqlite3DLL, "sqlite3_value_text16be")
    sqlite3_value_text16le        = GetProcAddress_(sqlite3DLL, "sqlite3_value_text16le")
    sqlite3_value_type            = GetProcAddress_(sqlite3DLL, "sqlite3_value_type")
    sqlite3_vmprintf              = GetProcAddress_(sqlite3DLL, "sqlite3_vmprintf")

    If sqlite3_get_table = 0
      LastMessage = "Couldn't init 'sqlite3_get_table'"
      ProcedureReturn #False
    EndIf

    If sqlite3_free_table = 0
      LastMessage = "Couldn't init 'sqlite3_free_table'"
      ProcedureReturn #False
    EndIf

    If sqlite3_open = 0
      LastMessage = "Couldn't init 'sqlite3_open'"
      ProcedureReturn #False
    EndIf

    If sqlite3_close = 0
      LastMessage = "Couldn't init 'sqlite3_close'"
      ProcedureReturn #False
    EndIf

    If sqlite3_exec = 0
      LastMessage = "Couldn't init 'sqlite3_exec'"
      ProcedureReturn #False
    EndIf

    LibIsInitializedWithIncludedDLL = #False

    ProcedureReturn #True

  Else

    LastMessage = "General Error - Couldn't open '"+sSQLiteDLLPath+"'"
    ProcedureReturn #False

  EndIf

EndProcedure

ProcedureDLL.l SQLite3_InitLibFromMemory(SQLiteDLLMemoryPointer.l)

  If SQLiteDLLMemoryPointer = 0
    LastMessage = "Invalid Pointer!"
    ProcedureReturn #False
  EndIf

  If sqlite3DLL
    SQLite3_End()
  EndIf

  sqlite3DLL = LoadLibraryM(SQLiteDLLMemoryPointer)

  If sqlite3DLL

    sqlite3_aggregate_context     = GetProcAddressM(sqlite3DLL, "sqlite3_aggregate_context")
    sqlite3_aggregate_count       = GetProcAddressM(sqlite3DLL, "sqlite3_aggregate_count")
    sqlite3_bind_blob             = GetProcAddressM(sqlite3DLL, "sqlite3_bind_blob")
    sqlite3_bind_double           = GetProcAddressM(sqlite3DLL, "sqlite3_bind_double")
    sqlite3_bind_int              = GetProcAddressM(sqlite3DLL, "sqlite3_bind_int")
    sqlite3_bind_int64            = GetProcAddressM(sqlite3DLL, "sqlite3_bind_int64")
    sqlite3_bind_null             = GetProcAddressM(sqlite3DLL, "sqlite3_bind_null")
    sqlite3_bind_parameter_count  = GetProcAddressM(sqlite3DLL, "sqlite3_bind_parameter_count")
    sqlite3_bind_parameter_index  = GetProcAddressM(sqlite3DLL, "sqlite3_bind_parameter_index")
    sqlite3_bind_parameter_name   = GetProcAddressM(sqlite3DLL, "sqlite3_bind_parameter_name")
    sqlite3_bind_text             = GetProcAddressM(sqlite3DLL, "sqlite3_bind_text")
    sqlite3_bind_text16           = GetProcAddressM(sqlite3DLL, "sqlite3_bind_text16")
    sqlite3_busy_handler          = GetProcAddressM(sqlite3DLL, "sqlite3_busy_handler")
    sqlite3_busy_timeout          = GetProcAddressM(sqlite3DLL, "sqlite3_busy_timeout")
    sqlite3_changes               = GetProcAddressM(sqlite3DLL, "sqlite3_changes")
    sqlite3_close                 = GetProcAddressM(sqlite3DLL, "sqlite3_close")
    sqlite3_collation_needed      = GetProcAddressM(sqlite3DLL, "sqlite3_collation_needed")
    sqlite3_collation_needed16    = GetProcAddressM(sqlite3DLL, "sqlite3_collation_needed16")
    sqlite3_column_blob           = GetProcAddressM(sqlite3DLL, "sqlite3_column_blob")
    sqlite3_column_bytes          = GetProcAddressM(sqlite3DLL, "sqlite3_column_bytes")
    sqlite3_column_bytes16        = GetProcAddressM(sqlite3DLL, "sqlite3_column_bytes16")
    sqlite3_column_count          = GetProcAddressM(sqlite3DLL, "sqlite3_column_count")
    sqlite3_column_decltype       = GetProcAddressM(sqlite3DLL, "sqlite3_column_decltype")
    sqlite3_column_decltype16     = GetProcAddressM(sqlite3DLL, "sqlite3_column_decltype16")
    sqlite3_column_double         = GetProcAddressM(sqlite3DLL, "sqlite3_column_double")
    sqlite3_column_int            = GetProcAddressM(sqlite3DLL, "sqlite3_column_int")
    sqlite3_column_int64          = GetProcAddressM(sqlite3DLL, "sqlite3_column_int64")
    sqlite3_column_name           = GetProcAddressM(sqlite3DLL, "sqlite3_column_name")
    sqlite3_column_name16         = GetProcAddressM(sqlite3DLL, "sqlite3_column_name16")
    sqlite3_column_text           = GetProcAddressM(sqlite3DLL, "sqlite3_column_text")
    sqlite3_column_text16         = GetProcAddressM(sqlite3DLL, "sqlite3_column_text16")
    sqlite3_column_type           = GetProcAddressM(sqlite3DLL, "sqlite3_column_type")
    sqlite3_commit_hook           = GetProcAddressM(sqlite3DLL, "sqlite3_commit_hook")
    sqlite3_complete              = GetProcAddressM(sqlite3DLL, "sqlite3_complete")
    sqlite3_complete16            = GetProcAddressM(sqlite3DLL, "sqlite3_complete16")
    sqlite3_create_collation      = GetProcAddressM(sqlite3DLL, "sqlite3_create_collation")
    sqlite3_create_collation16    = GetProcAddressM(sqlite3DLL, "sqlite3_create_collation16")
    sqlite3_create_function       = GetProcAddressM(sqlite3DLL, "sqlite3_create_function")
    sqlite3_create_function16     = GetProcAddressM(sqlite3DLL, "sqlite3_create_function16")
    sqlite3_data_count            = GetProcAddressM(sqlite3DLL, "sqlite3_data_count")
    sqlite3_errcode               = GetProcAddressM(sqlite3DLL, "sqlite3_errcode")
    sqlite3_errmsg                = GetProcAddressM(sqlite3DLL, "sqlite3_errmsg")
    sqlite3_errmsg16              = GetProcAddressM(sqlite3DLL, "sqlite3_errmsg16")
    sqlite3_exec                  = GetProcAddressM(sqlite3DLL, "sqlite3_exec")
    sqlite3_finalize              = GetProcAddressM(sqlite3DLL, "sqlite3_finalize")
    sqlite3_free                  = GetProcAddressM(sqlite3DLL, "sqlite3_free")
    sqlite3_free_table            = GetProcAddressM(sqlite3DLL, "sqlite3_free_table")
    sqlite3_get_auxdata           = GetProcAddressM(sqlite3DLL, "sqlite3_get_auxdata")
    sqlite3_get_table             = GetProcAddressM(sqlite3DLL, "sqlite3_get_table")
    sqlite3_interrupt             = GetProcAddressM(sqlite3DLL, "sqlite3_interrupt")
    sqlite3_last_insert_rowid     = GetProcAddressM(sqlite3DLL, "sqlite3_last_insert_rowid")
    sqlite3_libversion            = GetProcAddressM(sqlite3DLL, "sqlite3_libversion")
    sqlite3_mprintf               = GetProcAddressM(sqlite3DLL, "sqlite3_mprintf")
    sqlite3_open                  = GetProcAddressM(sqlite3DLL, "sqlite3_open")
    sqlite3_open16                = GetProcAddressM(sqlite3DLL, "sqlite3_open16")
    sqlite3_prepare               = GetProcAddressM(sqlite3DLL, "sqlite3_prepare")
    sqlite3_prepare16             = GetProcAddressM(sqlite3DLL, "sqlite3_prepare16")
    sqlite3_progress_handler      = GetProcAddressM(sqlite3DLL, "sqlite3_progress_handler")
    sqlite3_reset                 = GetProcAddressM(sqlite3DLL, "sqlite3_reset")
    sqlite3_result_blob           = GetProcAddressM(sqlite3DLL, "sqlite3_result_blob")
    sqlite3_result_double         = GetProcAddressM(sqlite3DLL, "sqlite3_result_double")
    sqlite3_result_error          = GetProcAddressM(sqlite3DLL, "sqlite3_result_error")
    sqlite3_result_error16        = GetProcAddressM(sqlite3DLL, "sqlite3_result_error16")
    sqlite3_result_int            = GetProcAddressM(sqlite3DLL, "sqlite3_result_int")
    sqlite3_result_int64          = GetProcAddressM(sqlite3DLL, "sqlite3_result_int64")
    sqlite3_result_null           = GetProcAddressM(sqlite3DLL, "sqlite3_result_null")
    sqlite3_result_text           = GetProcAddressM(sqlite3DLL, "sqlite3_result_text")
    sqlite3_result_text16         = GetProcAddressM(sqlite3DLL, "sqlite3_result_text16")
    sqlite3_result_text16be       = GetProcAddressM(sqlite3DLL, "sqlite3_result_text16be")
    sqlite3_result_text16le       = GetProcAddressM(sqlite3DLL, "sqlite3_result_text16le")
    sqlite3_result_value          = GetProcAddressM(sqlite3DLL, "sqlite3_result_value")
    sqlite3_set_authorizer        = GetProcAddressM(sqlite3DLL, "sqlite3_set_authorizer")
    sqlite3_set_auxdata           = GetProcAddressM(sqlite3DLL, "sqlite3_set_auxdata")
    sqlite3_snprintf              = GetProcAddressM(sqlite3DLL, "sqlite3_snprintf")
    sqlite3_step                  = GetProcAddressM(sqlite3DLL, "sqlite3_step")
    sqlite3_total_changes         = GetProcAddressM(sqlite3DLL, "sqlite3_total_changes")
    sqlite3_trace                 = GetProcAddressM(sqlite3DLL, "sqlite3_trace")
    sqlite3_user_data             = GetProcAddressM(sqlite3DLL, "sqlite3_user_data")
    sqlite3_value_blob            = GetProcAddressM(sqlite3DLL, "sqlite3_value_blob")
    sqlite3_value_bytes           = GetProcAddressM(sqlite3DLL, "sqlite3_value_bytes")
    sqlite3_value_bytes16         = GetProcAddressM(sqlite3DLL, "sqlite3_value_bytes16")
    sqlite3_value_double          = GetProcAddressM(sqlite3DLL, "sqlite3_value_double")
    sqlite3_value_int             = GetProcAddressM(sqlite3DLL, "sqlite3_value_int")
    sqlite3_value_int64           = GetProcAddressM(sqlite3DLL, "sqlite3_value_int64")
    sqlite3_value_text            = GetProcAddressM(sqlite3DLL, "sqlite3_value_text")
    sqlite3_value_text16          = GetProcAddressM(sqlite3DLL, "sqlite3_value_text16")
    sqlite3_value_text16be        = GetProcAddressM(sqlite3DLL, "sqlite3_value_text16be")
    sqlite3_value_text16le        = GetProcAddressM(sqlite3DLL, "sqlite3_value_text16le")
    sqlite3_value_type            = GetProcAddressM(sqlite3DLL, "sqlite3_value_type")
    sqlite3_vmprintf              = GetProcAddressM(sqlite3DLL, "sqlite3_vmprintf")

    If sqlite3_get_table = 0
      LastMessage = "Couldn't init 'sqlite3_get_table'"
      ProcedureReturn #False
    EndIf

    If sqlite3_free_table = 0
      LastMessage = "Couldn't init 'sqlite3_free_table'"
      ProcedureReturn #False
    EndIf

    If sqlite3_open = 0
      LastMessage = "Couldn't init 'sqlite3_open'"
      ProcedureReturn #False
    EndIf

    If sqlite3_close = 0
      LastMessage = "Couldn't init 'sqlite3_close'"
      ProcedureReturn #False
    EndIf

    If sqlite3_exec = 0
      LastMessage = "Couldn't init 'sqlite3_exec'"
      ProcedureReturn #False
    EndIf

    LibIsInitializedWithIncludedDLL = #True

    ProcedureReturn #True

  Else

    LastMessage = "General Error - Couldn't load library from address " + Str(SQLiteDLLMemoryPointer)
    ProcedureReturn #False

  EndIf

EndProcedure

ProcedureDLL.l SQLite3_CreateDatabase(sDataBase.s, bolOverwrite.l)

  If FileSize(sDataBase) = -1  ; DB noch nicht vorhanden

    ; Erstellen

    hResult = CallCFunctionFast(sqlite3_open, sDataBase, @sqlite3_dbHandle)

    If hResult = #SQLITE3_OK
      LastMessage = "OK"
      ProcedureReturn sqlite3_dbHandle
    Else
      LastMessage = "Couldn't create database"
      ProcedureReturn #False
    EndIf

  Else

    If bolOverwrite = 1 ; DB vorhanden, aber berschreiben

      ; berschreiben

      DeleteFile(sDataBase)

      hResult = CallCFunctionFast(sqlite3_open, sDataBase, @sqlite3_dbHandle)

      If hResult = #SQLITE3_OK
        LastMessage = "OK"
        ProcedureReturn sqlite3_dbHandle
      Else
        LastMessage = "Couldn't create database"
        ProcedureReturn #False
      EndIf

    EndIf

  EndIf

  ProcedureReturn #False

EndProcedure

ProcedureDLL.l SQLite3_OpenDatabase(sDataBase.s)

  If FileSize(sDataBase) <> -1

    hResult = CallCFunctionFast(sqlite3_open, sDataBase, @sqlite3_dbHandle)

    If hResult = #SQLITE3_OK
      LastMessage = "OK"
      ProcedureReturn sqlite3_dbHandle
    Else
      CallCFunctionFast(sqlite3_errmsg, @ReturnValue)
      LastMessage = PeekS(ReturnValue)
      ProcedureReturn #False
    EndIf

  Else

    LastMessage = "Database doesn't exists"
    ProcedureReturn #False

  EndIf

EndProcedure

ProcedureDLL.l SQLite3_CloseDatabase(lDataBaseHandle.l)

  If lDataBaseHandle = 0
    LastMessage = "Wrong Database-Handle"
    ProcedureReturn
  EndIf

  hResult = CallCFunctionFast(sqlite3_close, lDataBaseHandle)

  If hResult = #SQLITE3_OK
    LastMessage = "OK"
    ProcedureReturn #True
  Else
    LastMessage = "Couldn't close Database"
    ProcedureReturn #False
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_Execute(sSQLQuery.s, lDataBaseHandle.l)

  If lDataBaseHandle = 0
    LastMessage = "Wrong Database-Handle"
    ProcedureReturn
  EndIf
  
  Define sSQLQuery_U.s
  sSQLQuery_U = Space(Len(sSQLQuery) * 2)
  PokeS(@sSQLQuery_U, sSQLQuery, #PB_Any, #PB_UTF8)
  hResult = CallCFunctionFast(sqlite3_exec, lDataBaseHandle, sSQLQuery_U, #Null, #Null, @ReturnValue)

  If hResult = #SQLITE3_OK
    LastMessage = "OK"
    ProcedureReturn #True
  Else
    CallCFunctionFast(sqlite3_errmsg, @ReturnValue)
    LastMessage = PeekS(ReturnValue)
    ProcedureReturn #False
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_GetRecordset(sSQLQuery.s, lDataBaseHandle.l, *Recordset.SQLite3_Recordset)

  If lDataBaseHandle = 0
    LastMessage = "Wrong Database-Handle"
    ProcedureReturn
  EndIf

  *Recordset\BOF        = #True
  *Recordset\EOF        = #True
  *Recordset\Handle     = 0
  *Recordset\Rows       = 0
  *Recordset\Cols       = 0
  *Recordset\CurrentPos = 0
  *Recordset\sValue     = ""
  
  Define sSQLQuery_U.s
  sSQLQuery_U = Space(Len(sSQLQuery) * 2)
  PokeS(@sSQLQuery_U, sSQLQuery, #PB_Any, #PB_UTF8)
  hResult = CallCFunctionFast(sqlite3_get_table, lDataBaseHandle, sSQLQuery_U, @lResultsPtr, @lRows, @lCols, @ReturnValue)

  If hResult = #SQLITE3_OK

    *Recordset\Handle = lResultsPtr
    *Recordset\Rows = lRows
    *Recordset\Cols = lCols
    *Recordset\CurrentPos = 1
    *Recordset\BOF = #True

    If lRows > 0
      *Recordset\EOF = #False
    Else
      *Recordset\EOF = #True
    EndIf

    LastMessage = "OK"

    ProcedureReturn @Recordset

  Else

    CallCFunctionFast(sqlite3_errmsg, @ReturnValue)
    LastMessage = PeekS(ReturnValue)
    ProcedureReturn #False

  EndIf

EndProcedure

ProcedureDLL.l SQLite3_GetFieldIndexByName(sFieldname.s, *Recordset.SQLite3_Recordset)

  If *Recordset\Handle = 0
    LastMessage = "Wrong Recordset-Handle"
    ProcedureReturn
  EndIf

  If *Recordset\Handle <> 0

    Address.l  = *Recordset\Handle

    ; Search named Column
    For Col.l  = 0 To *Recordset\Cols - 1
      If sFieldname = PeekS(PeekL(Address + Col  * 4))
        LastMessage = "OK"
        ProcedureReturn Col
      EndIf
    Next

  EndIf

  LastMessage = "No such Columnname: '" + sFieldname + "'"
  ProcedureReturn -1

EndProcedure

ProcedureDLL.l SQLite3_GetRecordsetValueByName(sFieldname.s, *Recordset.SQLite3_Recordset)

  If *Recordset\Handle = 0
    LastMessage = "Wrong Recordset-Handle"
    ProcedureReturn
  EndIf

  myCol = SQLite3_GetFieldIndexByName(sFieldname, *Recordset)

  If myCol > -1

    LastMessage = "OK"
    ValueAddress = PeekL(*Recordset\Handle + (myCol * 4) + (*Recordset\CurrentPos * *Recordset\Cols) * 4)
    If ValueAddress
      *Recordset\sValue = PeekS(ValueAddress, #PB_Any, #PB_UTF8)
    Else
      *Recordset\sValue = ""
    EndIf
    ProcedureReturn #True

  Else

    LastMessage = "No such Columnname: '" + sFieldname + "'"
    *Recordset\sValue = ""
    ProcedureReturn #False

  EndIf

EndProcedure

ProcedureDLL.l SQLite3_GetRecordsetValueByIndex(lIndex.l, *Recordset.SQLite3_Recordset)

  If *Recordset\Handle = 0
    LastMessage = "Wrong Recordset-Handle"
    ProcedureReturn
  EndIf

  If lIndex < 0 Or lIndex > *Recordset\Cols

    LastMessage = "No such Columnindex: '" + Str(lIndex) + "'"
    *Recordset\sValue = ""
    ProcedureReturn #False

  Else

    LastMessage = "OK"
    ValueAddress = PeekL(*Recordset\Handle + (lIndex * 4) + (*Recordset\CurrentPos * *Recordset\Cols) * 4)
    If ValueAddress ; To avoid NULL-Values
      *Recordset\sValue = PeekS(ValueAddress, #PB_Any, #PB_UTF8)
    Else
      *Recordset\sValue = ""
    EndIf
    ProcedureReturn #True

  EndIf

EndProcedure

ProcedureDLL.l SQLite3_ReleaseRecordset(*Recordset.SQLite3_Recordset)
  If *Recordset\Handle <> 0
    CallCFunctionFast(sqlite3_free_table, *Recordset\Handle)
    *Recordset\Handle =0
  EndIf
  LastMessage = "OK"
EndProcedure

ProcedureDLL.l SQLite3_RecordsetMoveNext(*Recordset.SQLite3_Recordset)

  If *Recordset\Handle = 0
    LastMessage = "Wrong Recordset-Handle"
    ProcedureReturn
  EndIf

  *Recordset\BOF = #False
  *Recordset\EOF = #False

  *Recordset\CurrentPos = *Recordset\CurrentPos + 1

  If *Recordset\CurrentPos > *Recordset\Rows
    *Recordset\CurrentPos = *Recordset\Rows
    *Recordset\EOF = #True
  EndIf

  If *Recordset\CurrentPos = 1
    *Recordset\BOF = #True
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_RecordsetMovePrevious(*Recordset.SQLite3_Recordset)

  If *Recordset\Handle = 0
    LastMessage = "Wrong Recordset-Handle"
    ProcedureReturn
  EndIf

  *Recordset\BOF = #False
  *Recordset\EOF = #False

  *Recordset\CurrentPos = *Recordset\CurrentPos - 1

  If *Recordset\CurrentPos < 1
    *Recordset\CurrentPos = 1
    *Recordset\BOF = #True
  EndIf

  If *Recordset\CurrentPos = *Recordset\Rows
    *Recordset\EOF = #True
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_RecordsetMoveFirst(*Recordset.SQLite3_Recordset)

  If *Recordset\Handle = 0
    LastMessage = "Wrong Recordset-Handle"
    ProcedureReturn
  EndIf

  *Recordset\BOF = #True
  *Recordset\EOF = #False

  *Recordset\CurrentPos = 1

  If *Recordset\CurrentPos = *Recordset\Rows
    *Recordset\EOF = #True
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_RecordsetMoveLast(*Recordset.SQLite3_Recordset)

  If *Recordset\Handle = 0
    LastMessage = "Wrong Recordset-Handle"
    ProcedureReturn
  EndIf

  *Recordset\BOF = #False
  *Recordset\EOF = #True

  *Recordset\CurrentPos = *Recordset\Rows

  If *Recordset\CurrentPos = 1
    *Recordset\BOF = #True
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_LastInsertRowID(lDataBaseHandle.l)

  If lDataBaseHandle = 0
    LastMessage = "Wrong Database-Handle"
    ProcedureReturn
  EndIf

  ProcedureReturn CallCFunctionFast(sqlite3_last_insert_rowid, lDataBaseHandle)

  ; If hResult = #SQLITE3_OK
  ; LastMessage = "OK"
  ; ProcedureReturn #True
  ; Else
  ; CallCFunctionFast(sqlite3_errmsg, @ReturnValue)
  ; LastMessage = PeekS(ReturnValue)
  ; ProcedureReturn #False
  ; EndIf

EndProcedure

ProcedureDLL.l SQLite3_QueryReturnRows(sSQLQuery.s, lDataBaseHandle.l)

  If lDataBaseHandle = 0
    LastMessage = "Wrong Database-Handle"
    ProcedureReturn
  EndIf

  If sSQLQuery = ""
    LastMessage = "Query is empty"
    ProcedureReturn
  EndIf

  dRS.SQLite3_Recordset

  SQLite3_GetRecordset(sSQLQuery, lDataBaseHandle, @dRS)

  ReturnedRows.l = dRS\Rows

  SQLite3_ReleaseRecordset(@dRS)

  LastMessage = "OK"

  ProcedureReturn ReturnedRows

EndProcedure

ProcedureDLL.s SQLite3_GetLastMessage()
  ProcedureReturn LastMessage
EndProcedure

ProcedureDLL.s SQLite3_GetLibVersion()
  ProcedureReturn "SQLite3 Userlibrary V1.1 (21.10.2005)"
EndProcedure

ProcedureDLL.l SQLite3_Prepare(lDataBaseHandle.l, sSQLQuery.s)

  If lDataBaseHandle = 0
    LastMessage = "Wrong Database-Handle"
    ProcedureReturn
  EndIf

  If sSQLQuery = ""
    LastMessage = "SQL-Query is empty"
    ProcedureReturn
  EndIf
  
  Define sSQLQuery_U.s
  sSQLQuery_U = Space(Len(sSQLQuery) * 2)
  PokeS(@sSQLQuery_U, sSQLQuery, #PB_Any, #PB_UTF8)
  hResult = CallCFunctionFast(sqlite3_prepare, lDataBaseHandle, sSQLQuery_U, -1, @ReturnValue, #Null)

  SetLastMessage(hResult)

  If hResult = #SQLITE3_OK
    ProcedureReturn ReturnValue
  Else
    ProcedureReturn #False
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_Bind_Blob(lStatement.l, lIndex.l, lBinaryBuffer.l, lBinaryBufferLength.l)

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn
  EndIf

  If lIndex = 0
    LastMessage = "Index must be greater than 0"
    ProcedureReturn
  EndIf

  If lBinaryBuffer = 0
    LastMessage = "Binarybuffer is empty"
    ProcedureReturn
  EndIf

  If lBinaryBufferLength = 0
    LastMessage = "Binarybufferlength is empty"
    ProcedureReturn
  EndIf

  hResult = CallCFunctionFast(sqlite3_bind_blob, lStatement, lIndex, lBinaryBuffer, lBinaryBufferLength, -1)

  If hResult = #SQLITE3_OK
    LastMessage = "OK"
    ProcedureReturn #True
  Else
    CallCFunctionFast(sqlite3_errmsg, @ReturnValue)
    LastMessage = PeekS(ReturnValue)
    ProcedureReturn #False
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_Bind_Text(lStatement.l, lIndex.l, lTextBuffer.l)

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn
  EndIf

  If lIndex = 0
    LastMessage = "Index must be greater than 0"
    ProcedureReturn
  EndIf

  If lTextBuffer = 0
    LastMessage = "Textbuffer is empty"
    ProcedureReturn
  EndIf

  hResult = CallCFunctionFast(sqlite3_bind_text, lStatement, lIndex, lTextBuffer, -1, #SQLITE3_STATIC)

  SetLastMessage(hResult)

  If hResult = #SQLITE3_OK
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_Bind_Int(lStatement.l, lIndex.l, lInteger.l)

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn
  EndIf

  If lIndex = 0
    LastMessage = "Index must be greater than 0"
    ProcedureReturn
  EndIf

  hResult = CallCFunctionFast(sqlite3_bind_int, lStatement, lIndex, lInteger, #SQLITE3_STATIC)

  SetLastMessage(hResult)

  If hResult = #SQLITE3_OK
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_Column_Bytes(lStatement.l, lColumnIndex.l)

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn
  EndIf

  hResult = CallCFunctionFast(sqlite3_column_bytes, lStatement, lColumnIndex.l)

  LastMessage = "OK"
  ProcedureReturn hResult

EndProcedure

ProcedureDLL.l SQLite3_Column_Count(lStatement.l) ; Return the number of columns in the result set returned by the prepared SQL statement

  ; Return the number of columns in the result set returned by the prepared SQL statement.
  ; This routine returns 0 if lStatement is an SQL statement that does not return data (for example an UPDATE).

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn
  EndIf

  hResult = CallCFunctionFast(sqlite3_column_count, lStatement, 0)

  LastMessage = "OK"
  ProcedureReturn hResult

EndProcedure

ProcedureDLL.l SQLite3_Column_Type(lStatement.l, lColumnIndex.l) ; Return the datatype of a column

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn #False
  EndIf

  LastMessage = "OK"

  ProcedureReturn CallCFunctionFast(sqlite3_column_type, lStatement, lColumnIndex)

EndProcedure

ProcedureDLL.s SQLite3_Column_Name(lStatement.l, lColumnIndex.l) ; Return the name of a column

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn ""
  EndIf

  ReturnValue = CallCFunctionFast(sqlite3_column_name, lStatement, lColumnIndex)

  If ReturnValue
    LastMessage = "OK"
    ProcedureReturn PeekS(ReturnValue)
  Else
    LastMessage = "Couldn't get columnname"
    ProcedureReturn ""
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_Column_Blob(lStatement.l, lColumnIndex.l) ; Return the address of a blob

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn #False
  EndIf

  LastMessage = "OK"

  ProcedureReturn CallCFunctionFast(sqlite3_column_blob, lStatement, lColumnIndex)

EndProcedure

ProcedureDLL.l SQLite3_Column_Text(lStatement.l, lColumnIndex.l) ; Return the address of a text

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn #False
  EndIf

  LastMessage = "OK"

  ProcedureReturn CallCFunctionFast(sqlite3_column_text, lStatement, lColumnIndex)

EndProcedure

ProcedureDLL.l SQLite3_Column_Int(lStatement.l, lColumnIndex.l) ; Return the address of a text

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn #False
  EndIf

  LastMessage = "OK"

  ProcedureReturn CallCFunctionFast(sqlite3_column_int, lStatement, lColumnIndex)

EndProcedure

ProcedureDLL.l SQLite3_Step(lStatement.l)

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn
  EndIf

  hResult = CallCFunctionFast(sqlite3_step, lStatement)

  SetLastMessage(hResult)

  If hResult = #SQLITE3_OK Or hResult = #SQLITE3_ROW Or hResult = #SQLITE3_DONE
    ProcedureReturn hResult
  Else
    ProcedureReturn #False
  EndIf

EndProcedure

ProcedureDLL.l SQLite3_Finalize(lStatement.l)

  If lStatement = 0
    LastMessage = #ERROR_STATEMENT_IS_EMPTY$
    ProcedureReturn
  EndIf

  hResult = CallCFunctionFast(sqlite3_finalize, lStatement)

  SetLastMessage(hResult)

  If hResult = #SQLITE3_OK
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf

EndProcedure

Procedure SetLastMessage(hResult)

  CallCFunctionFast(sqlite3_errmsg, @ReturnValue)

  If ReturnValue

    LastMessage = PeekS(ReturnValue)

  Else

    Select hResult

      Case #SQLITE3_OK
        LastMessage = "OK"
      Case #SQLITE3_ERROR        ; SQL error Or missing database
        LastMessage = "SQL error Or missing database"
      Case #SQLITE3_INTERNAL     ; An internal logic error in SQLite
        LastMessage = "An internal logic error in SQLite"
      Case #SQLITE3_PERM         ; Access permission denied
        LastMessage = "Access permission denied"
      Case #SQLITE3_ABORT        ; Callback routine requested An abort
        LastMessage = "Callback routine requested An abort"
      Case #SQLITE3_BUSY         ; The database file is locked
        LastMessage = "The database file is locked"
      Case #SQLITE3_LOCKED       ; A table in The database is locked
        LastMessage = "A table in The database is locked"
      Case #SQLITE3_NOMEM        ; A malloc() failed
        LastMessage = "A malloc() failed"
      Case #SQLITE3_READONLY     ; Attempt To write A readonly database
        LastMessage = "Attempt To write A readonly database"
      Case #SQLITE3_INTERRUPT    ; Operation terminated by SQLite_Interrupt()
        LastMessage = "Operation terminated by SQLite_Interrupt()"
      Case #SQLITE3_IOERR        ; Some kind of disk I/O error occurred
        LastMessage = "Some kind of disk I/O error occurred"
      Case #SQLITE3_CORRUPT      ; The database disk image is malformed
        LastMessage = "The database disk image is malformed"
      Case #SQLITE3_NOTFOUND     ; (internal Only) table Or record not found
        LastMessage = "(internal Only) table Or record not found"
      Case #SQLITE3_FULL         ; Insertion failed because database is full
        LastMessage = "Insertion failed because database is full"
      Case #SQLITE3_CANTOPEN     ; Unable To open The database file
        LastMessage = "Unable To open The database file"
      Case #SQLITE3_PROTOCOL     ; database lock protocol error
        LastMessage = "database lock protocol error"
      Case #SQLITE3_EMPTY        ; (internal Only) database table is empty
        LastMessage = "(internal Only) database table is empty"
      Case #SQLITE3_SCHEMA       ; The database schema changed
        LastMessage = "he database schema changed"
      Case #SQLITE3_TOOBIG       ; Too much Data For one Row of A table
        LastMessage = "Too much Data For one Row of A table"
      Case #SQLITE3_CONStraint   ; abort due To contraint violation
        LastMessage = "abort due To contraint violation"
      Case #SQLITE3_MISMATCH     ; Data type mismatch
        LastMessage = "Data type mismatch"
      Case #SQLITE3_MISUSE       ; Library used incorrectly
        LastMessage = "Library used incorrectly"
      Case #SQLITE3_NOLFS        ; Uses OS features not supported on host
        LastMessage = "Uses OS features not supported on host"
      Case #SQLITE3_AUTH         ; Authorization denied
        LastMessage = "Authorization denied"
      Case #SQLITE3_ROW          ; sqlite_step() has another Row ready
        LastMessage = "sqlite_step() has another Row ready"
      Case #SQLITE3_DONE         ; sqlite_step() has finished executing
        LastMessage = "sqlite_step() has finished executing"
      Default
        LastMessage = "Unknown Error (No. " + Str(hResult) + ")"
    EndSelect

  EndIf

EndProcedure
; IDE Options = PureBasic v4.02 (Windows - x86)
; CursorPosition = 857
; FirstLine = 143
; Folding = AEASAA-
; EnableXP
; HideErrorLog